@@ -14,17 +14,21 @@
  * Boston, MA 021110-1307, USA.
  */
 
+#include "kerncompat.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <getopt.h>
-
-#include "volumes.h"
+#include <stdbool.h>
+#include <strings.h>
+#include "kernel-shared/volumes.h"
 #include "crypto/crc32c.h"
-#include "cmds/commands.h"
 #include "common/utils.h"
+#include "common/string-utils.h"
 #include "common/help.h"
 #include "common/box.h"
+#include "common/messages.h"
+#include "cmds/commands.h"
 
 static const char * const btrfs_cmd_group_usage[] = {
 	"btrfs [--help] [--version] [--format <format>] [-v|--verbose] [-q|--quiet] <group> [<group>...] <command> [<args>]",
@@ -105,8 +109,7 @@ static void check_output_format(const struct cmd_struct *cmd)
 		return;
 
 	if (!(cmd->flags & bconf.output_format & CMD_FORMAT_MASK)) {
-		fprintf(stderr,
-			"ERROR: output format %s is unsupported for this command\n",
+		error("output format %s is unsupported for this command",
 			output_format_name(bconf.output_format));
 		exit(1);
 	}
@@ -330,21 +333,27 @@ static void handle_special_globals(int shift, int argc, char **argv)
 
 static const struct cmd_group btrfs_cmd_group = {
 	btrfs_cmd_group_usage, btrfs_cmd_group_info, {
-		&cmd_struct_subvolume,
-		&cmd_struct_filesystem,
+		/* Keep subcommands alphabetically sorted */
 		&cmd_struct_balance,
-		&cmd_struct_device,
-		&cmd_struct_scrub,
 		&cmd_struct_check,
-		&cmd_struct_rescue,
-		&cmd_struct_restore,
+		&cmd_struct_device,
+		&cmd_struct_filesystem,
 		&cmd_struct_inspect,
 		&cmd_struct_property,
-		&cmd_struct_send,
-		&cmd_struct_receive,
-		&cmd_struct_quota,
 		&cmd_struct_qgroup,
+		&cmd_struct_quota,
+		&cmd_struct_receive,
+#if EXPERIMENTAL
+		&cmd_struct_reflink,
+#endif
 		&cmd_struct_replace,
+		&cmd_struct_rescue,
+		&cmd_struct_restore,
+		&cmd_struct_scrub,
+		&cmd_struct_send,
+		&cmd_struct_subvolume,
+
+		/* Help and version stay last */
 		&cmd_struct_help,
 		&cmd_struct_version,
 		NULL
